function [V, converged, i] = newtonpf(Ybus, Sbus, V0, ref, pv, pq, tol, max_it, verbose)
%NEWTONPF  Solves the power flow using a full Newton-Raphson method.
%   [V, converged, i] = newtonpf(Ybus, Sbus, V0, ref, pv, pq, ...
%                                            tol, max_it, verbose)
%   solves for bus voltages given the full system admittance matrix (for
%   all buses), the complex bus power injection vector (for all buses),
%   the initial vector of complex bus voltages, and column vectors with
%   the lists of bus indices for the swing bus, PV buses, and PQ buses,
%   respectively. The bus voltage vector contains the set point for
%   generator (including ref bus) buses, and the reference angle of the
%   swing bus, as well as an initial guess for remaining magnitudes and
%   angles. The final power mismatch for each bus will be smaller than tol,
%   unless max_it iterations are performed before convergence is achieved.
%   Returns the final complex voltages, a flag which indicates whether
%   it converged or not, and the number of iterations performed.

%   MATPOWER Version 1.0
%   by Ray Zimmerman, PSERC Cornell    9/16/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% default arguments
if nargin < 9
	verbose = 0;
	if nargin < 8
		max_it = 10;
		if nargin < 7
			tol = 1e-6;
		end
	end
end

%% initialize
j = sqrt(-1);
converged = 0;
i = 0;
V = V0;
Va = angle(V);
Vm = abs(V);

%% set up indexing for updating V
npv	= length(pv);
npq	= length(pq);
j1 = 1;			j2 = npv;			%% j1:j2 - V angle of pv buses
j3 = j2 + 1;	j4 = j2 + npq;		%% j3:j4 - V angle of pq buses
j5 = j4 + 1;	j6 = j4 + npq;		%% j5:j6 - V mag of pq buses

%% evaluate F(x0)
mis = V .* conj(Ybus * V) - Sbus;
F = [	real(mis(pv));
		real(mis(pq));
		imag(mis(pq))	];

%% check tolerance
normF = norm(F, inf);
if verbose
	fprintf('\nmax( F(x%d) ) = %.2e', i, normF);
end
if normF < tol
	converged = 1;
	if verbose
		fprintf('\nConverged!\n');
	end
end

%% do Newton-Raphson iterations
while (~converged & i < max_it)
	%% update iteration counter
	i = i + 1;
	
	%% evaluate Jacobian
	[dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V);
	J = [
		real(dSbus_dVa(pv, pv)), real(dSbus_dVa(pv, pq)), real(dSbus_dVm(pv, pq));
		real(dSbus_dVa(pq, pv)), real(dSbus_dVa(pq, pq)), real(dSbus_dVm(pq, pq));
		imag(dSbus_dVa(pq, pv)), imag(dSbus_dVa(pq, pq)), imag(dSbus_dVm(pq, pq))
	];

	%% compute update step
	dx = -(J \ F);

	%% update voltage
	Va(pv) = Va(pv) + dx(j1:j2);
	Va(pq) = Va(pq) + dx(j3:j4);
	Vm(pq) = Vm(pq) + dx(j5:j6);
	V = Vm .* exp(j * Va);

	%% evalute F(x)
	mis = V .* conj(Ybus * V) - Sbus;
	F = [	real(mis(pv));
			real(mis(pq));
			imag(mis(pq))	];

	%% check for convergence
	normF = norm(F, inf);
	if verbose
		fprintf('\nmax( F(x%d) ) = %.2e', i, normF);
	end
	if normF < tol
		converged = 1;
		if verbose
			fprintf('\nConverged in %d iterations.\n', i);
		end
	end
end

if verbose
	if ~converged
		fprintf('\nDid not converge in %d iterations.\n', i);
	end
end
